/*******************************************************************************

Author : Mauro Laurenti
Version : 1.2
Created on Date : 19/03/2011
Last update     : 15/12/2014
 
CopyRight 2006-2014 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

********************************************************
PURPOSES
********************************************************

These functions allow the user to read/write to the data
port of the I2C I/O extender PCF8574. 
Either PCF8574 or PCF8574A can be used. The user must use the 
proper address calling the functions.

PCF8574  Address : 0100xxx0
PCF8574A Address : 0111xxx0

xxx = A2 A1 A0 (External Address Pins)

*******************************************************************************/

#ifndef PCF8574_H
#define PCF8574_H


#ifdef __XC8
    #include <xc.h>
        #ifndef _PIC18
            #error The PCF8574 Library supports only PIC18 devices
        #endif
#endif

#include <i2c.h>

/**
 * This function initialize the I2C module accordingly to the clock and baud rate.
 *
 *
 * @param crystal_frequency_MHz Clock frequency expressed in MHz 
 *
 * @param baud_rate_KHz Baud rate expressed in KHz
 *
 * @return 
 *
 * @note  If other devices are connected on the I2C bus the baud rate would be changed by
 * the function. Among several calls from different device initializations, only the last one will be
 * reflected in the I2C module configuration. 
 *
 */
void PCF8574_initialize (unsigned char crystal_frequency_MHz, unsigned int baud_rate_KHz);
#define initialize_PCF8574 PCF8574_initialize



/**
 * This function writes to the data Port of the I2C I/O expander.
 *
 *
 * @param control Is the address of the PCF8574 (Internal Address + Pin address)
 *
 * @param data Byte to write [min: 0, max: 255]
 *
 * @return status 1: The byte has been properly written
 *               -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 *
 * @note    PCF8574  Address : 0100xxx0
 *          PCF8574A Address : 0111xxx0
 *
 *          xxx = A2 A1 A0 (External Address Pins)
 */

signed char PCF8574_write_data (unsigned char control, unsigned char data );
#define write_data_PCF8574 PCF8574_write_data

/**
 * This function reads to the data Port of the I2C I/O expander.
 *
 *
 * @param control Is the address of the PCF8574 (Internal Address + Pin address
 *
 * @param data Address where the data is written into.
 *
 * @return status 1: The byte has been properly written
 *       	 -1: Bus Collision error
 *               -2: Not Ack error condition
 *               -3: Write collision
 *
 * @note    PCF8574  Address : 0100xxx0
 *          PCF8574A Address : 0111xxx0
 *
 *          xxx = A2 A1 A0 (External Address Pins) 
 */

signed char PCF8574_read_data(unsigned char control, unsigned char *data);
#define read_data_PCF8574 PCF8574_read_data


#endif
