/*******************************************************************************

Author : Dario Di Turi
Version : 1.2
Date : 08/12/2012

Last Update: Mauro Laurenti
Last Update: 15/12/2014

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Dario Di Turi (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.
*************************************************************/

#ifdef __XC8
    #include <xc.h>
#endif

#include "AD9833.h"

unsigned int  K;
unsigned int  register_word;
unsigned int  tuning_freq;


//*********************************************
//             write_register
//*********************************************	
void AD9833_write_data(unsigned int data){

	OpenSPI(SPI_FOSC_64,MODE_10,SMPMID);
	
	//FSYNC low, load word into the device 
	FSYNC=0; 

	//MSB data
	while(WriteSPI(data>>8));
	//LSB data
	while(WriteSPI(data));
	
	//FSYNC high, load word completed 
	FSYNC=1; 
	CloseSPI();

}


//*********************************************
//         compose_AD9833_control_word
//*********************************************	
unsigned int AD9833_compose_control_word(unsigned char function_type){


	// Write frequency register
	if(function_type==0){
		register_word=register_word | 0b0010000000000000; 
		return(register_word);
	}
	
	// Sinusoidal waveform
	if(function_type==1){
		register_word=register_word & 0b1111111111000000;	
		return(register_word);
	}
	
	// Triangular waveform
	if(function_type==2){
		register_word=register_word & 0b1111111111000000;
		register_word=register_word | 0b0000000000000010;
		return(register_word);
	}
	
	// Square waveform
	if(function_type==3){
		register_word=register_word & 0b1111111111000000;	
		register_word=register_word | 0b0000000000101000;
		return(register_word);
		
	}
	// FREQ0 out
	if(function_type==4){
		register_word=register_word & 0b1111011111111111;	
		return(register_word);
		
	}
	// FREQ1 out
	if(function_type==5){
		register_word=register_word & 0b1111011111111111;	
		register_word=register_word | 0b0000100000000000;
		return(register_word);
		
	}	
	// PHASE0 out
	if(function_type==6){
		register_word=register_word & 0b1111101111111111;	
		return(register_word);
		
	}
	// PHASE1 out
	if(function_type==7){
		register_word=register_word & 0b1111101111111111;	
		register_word=register_word | 0b0000010000000000;
		return(register_word);
		
	}
	// Write LSB frequency register
	if(function_type==8){
		register_word=register_word & 0b0000111111111111;	
		return(register_word);
		
	}
	// Sleep mode
	if(function_type==9){
		register_word=register_word & 0b1111111100111111;	
		register_word=register_word | 0b0000010011000000;
		return(register_word);
		
	}
	// Power ON
	if(function_type==10){
		register_word=register_word & 0b1111111100111111;	
		return(register_word);
		
	}

        // This return is never executed but it removes a compiler warning.
        return (0);
}


//*********************************************
//               AD9833_initialize
//*********************************************	
void AD9833_initialize (unsigned char fmclk){

	register_word=0x0000;
	tuning_freq=0x0000;
	
	K=268435456/(fmclk*100000); //2^28/(fmclk*100000)
	
	AD9833_write_data (AD9833_RESET);
	AD9833_set_frequency (0, AD9833_REG0);
	AD9833_set_frequency (0, AD9833_REG1);
	AD9833_set_phase (0,AD9833_REG0);
	AD9833_set_phase (0,AD9833_REG1);
 
 
	AD9833_set_function (AD9833_FREQUENCY0_OUT);
	AD9833_set_function (AD9833_PHASE0_OUT);
	AD9833_set_function (AD9833_SIN);

}


//*********************************************
//           AD9833_set_frequency
//*********************************************	
void AD9833_set_frequency (unsigned long freq_value, unsigned char frequency_register){
 
	unsigned long freq_reg_value;
	unsigned int freq_reg_value_l,freq_reg_value_h;
	unsigned int control_word;

	freq_reg_value = (freq_value/10) * K; //2^28/(fmclk)
	
	freq_reg_value = freq_reg_value << 2;
	freq_reg_value_l = freq_reg_value;
	freq_reg_value_h = freq_reg_value >> 16;
 	freq_reg_value_l = freq_reg_value_l >> 2;
	
	// Save the LSB frequency register for the tuning function
	tuning_freq=freq_reg_value_l;
 
	// Write into FREQ0 register
	if(frequency_register==0){
 
		freq_reg_value_l = freq_reg_value_l & 0x7FFF; //0b0111,1111,1111,1111
		freq_reg_value_l = freq_reg_value_l | 0x4000; //0b0100,0000,0000,0000

		freq_reg_value_h = freq_reg_value_h & 0x7FFF;
		freq_reg_value_h = freq_reg_value_h | 0x4000;
	}
	
	// Write into FREQ1 register
	if(frequency_register==1){
		
		freq_reg_value_l = freq_reg_value_l & 0xBFFF; //0b1011,1111,1111,1111
		freq_reg_value_l = freq_reg_value_l | 0x8000; //0b1000,0000,0000,0000

		freq_reg_value_h = freq_reg_value_h & 0xBFFF;
		freq_reg_value_h = freq_reg_value_h | 0x8000;
	}
 
	control_word = AD9833_compose_control_word(0);
	AD9833_write_data (control_word);
	AD9833_write_data (freq_reg_value_l);
	AD9833_write_data (freq_reg_value_h);
 
}


//*********************************************
//           AD9833_tuning_frequency
//*********************************************	
void AD9833_tuning_frequency (unsigned char tuning_step, unsigned char direction,unsigned char frequency_register ){

	unsigned int control_word;
	
	//tuning DOWN
	if(direction==0){
		
		tuning_freq=tuning_freq-tuning_step;
	}
	//tuning UP	
	if(direction==1){
		
		tuning_freq=tuning_freq+tuning_step;
	}
	// Write into FREQ0 register
	if(frequency_register==0){
 
		tuning_freq = tuning_freq & 0x7FFF; //0b0111,1111,1111,1111
		tuning_freq = tuning_freq | 0x4000; //0b0100,0000,0000,0000

	}
	
	// Write into FREQ1 register
	if(frequency_register==1){
		
		tuning_freq = tuning_freq & 0xBFFF; //0b1011,1111,1111,1111
		tuning_freq = tuning_freq | 0x8000; //0b1000,0000,0000,0000

	}
	
	control_word = AD9833_compose_control_word(8);
	AD9833_write_data (control_word);
	AD9833_write_data (tuning_freq);

}

//*********************************************
//           AD9833_set_phase
//*********************************************	
void AD9833_set_phase (unsigned long phase_value, unsigned char phase_register){

	unsigned int phase_reg;
	
	phase_value=(phase_value*1137)/1000;
	
	phase_reg=phase_value;
	
	// Write into PHASE0 register
	if(phase_register==0){
		
		phase_reg=phase_reg & 0xFFF;  //0b0000111111111111
		phase_reg=phase_reg | 0xC000; //0b1100000000000000
	}
	
	// Write into PHASE1 register
	if(phase_register==1){
		
		phase_reg=phase_reg & 0xFFF;  //0b0000111111111111
		phase_reg=phase_reg | 0xE000; //0b1110000000000000
	}	
 
	AD9833_write_data (phase_reg);
 
}

//*********************************************
//            AD9833_set_function
//*********************************************	
void AD9833_set_function (unsigned char function_type){

	unsigned int function;

	switch(function_type){
	
		case AD9833_SIN:
		function = AD9833_compose_control_word(1);
		AD9833_write_data (function);
		break;
		
		case AD9833_TRIANGLE:
		function = AD9833_compose_control_word(2);
		AD9833_write_data (function);
		break;
		
		case AD9833_SQUARE:
		function = AD9833_compose_control_word(3);
		AD9833_write_data (function);
		break;
		
		case AD9833_FREQUENCY0_OUT:
		function = AD9833_compose_control_word(4);
		AD9833_write_data (function);
		break;
		
		case AD9833_FREQUENCY1_OUT:
		function = AD9833_compose_control_word(5);
		AD9833_write_data (function);
		break;
		
		case AD9833_PHASE0_OUT:
		function = AD9833_compose_control_word(6);
		AD9833_write_data (function);
		break;

		case AD9833_PHASE1_OUT:
		function = AD9833_compose_control_word(7);
		AD9833_write_data (function);
		break;
		
		case AD9833_SLEEP_MODE:
		function = AD9833_compose_control_word(9);
		AD9833_write_data (function);
		break;
		
		case AD9833_POWER_ON:
		function = AD9833_compose_control_word(10);
		AD9833_write_data (function);
		break;
	}
}




