/*******************************************************************************

Author : Mauro Laurenti
Version : 1.2
Created on Date : 26/05/2012
Last update     : 15/12/2014

CopyRight 2006-2014 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/

#ifdef __XC8
    #include <xc.h>
#endif

#include "GLCD_KS0108B.h"


unsigned char font_width = 0;
unsigned char font_height = 0;


//*********************************************
//             GLCD_wait_busy_status
//*********************************************
void GLCD_wait_busy_status (void){

	unsigned char busy_flag = 1;

	while (busy_flag) {

		if (get_controller_status_GLCD () & GLCD_BUSY_FLAG) 
			busy_flag = 1;
		else
			busy_flag = 0;
	}
}


//*********************************************
//             GLCD_get_controller_status
//*********************************************
unsigned char GLCD_get_controller_status (void) {

	unsigned char controller_status; 

	GLCD_DATA_DIRECTION = GLCD_INPUT;

	GLCD_E = LOW_GLCD;
	NOP_DELAY;
	GLCD_R_W = HIGH_GLCD;
	GLCD_D_I = LOW_GLCD;
	NOP_DELAY;
	GLCD_E = HIGH_GLCD;
	NOP_DELAY;
	controller_status = GLCD_DATA_READ;
	
	GLCD_DATA_DIRECTION = GLCD_OUTPUT;

	return (controller_status);
}


//*********************************************
//             GLCD_write_command
//*********************************************
void GLCD_write_command (unsigned char command_to_send) {

	GLCD_E = LOW_GLCD;
	NOP_DELAY;
	NOP_DELAY;
	GLCD_R_W = LOW_GLCD;
	GLCD_D_I = LOW_GLCD;
	NOP_DELAY;

	GLCD_E = HIGH_GLCD;
	GLCD_DATA_WRITE = command_to_send;
	NOP_DELAY;
    GLCD_E = LOW_GLCD;

	NOP_DELAY;
	GLCD_R_W = HIGH_GLCD;
	GLCD_D_I = LOW_GLCD;

	NOP_DELAY;

	GLCD_E = HIGH_GLCD;
}


//*********************************************
//             GLCD_set_display
//*********************************************
void GLCD_set_display (unsigned char display_status){
	
	GLCD_CS1 = HIGH_GLCD;
	GLCD_CS2 = LOW_GLCD;

	wait_busy_status_GLCD ();
	if (display_status)
		write_command_GLCD (GLCD_COMM_DISPLAY_ON);
	else 
		write_command_GLCD (GLCD_COMM_DISPLAY_OFF);

	wait_busy_status_GLCD ();

	GLCD_CS1 = LOW_GLCD;
	GLCD_CS2 = HIGH_GLCD;

	wait_busy_status_GLCD ();
	if (display_status)
		write_command_GLCD (GLCD_COMM_DISPLAY_ON);
	else 
		write_command_GLCD (GLCD_COMM_DISPLAY_OFF);

	wait_busy_status_GLCD ();

	GLCD_CS1 = LOW_GLCD;
	GLCD_CS2 = LOW_GLCD;

}

//*********************************************
//             GLCD_set_page
//*********************************************
void GLCD_set_page (unsigned char controller_page) {

	wait_busy_status_GLCD ();
	write_command_GLCD (GLCD_COMM_PAGE_SELECT | controller_page);
	wait_busy_status_GLCD ();
}


//*********************************************
//             GLCD_set_address
//*********************************************
void GLCD_set_address (unsigned char controller_address){

	wait_busy_status_GLCD ();
	write_command_GLCD (GLCD_COMM_ADDRESS_SELECT | controller_address);
	wait_busy_status_GLCD ();
}


//*********************************************
//             GLCD_set_vertical_offset
//*********************************************
void GLCD_set_vertical_offset (unsigned char vertical_offset){

	GLCD_CS1 = HIGH_GLCD;
	GLCD_CS2 = LOW_GLCD;
    
    NOP_DELAY;

//	wait_busy_status_GLCD ();
	write_command_GLCD (GLCD_COMM_START_LINE | vertical_offset);
//	wait_busy_status_GLCD ();

	GLCD_CS1 = LOW_GLCD;
	GLCD_CS2 = HIGH_GLCD;
    
    NOP_DELAY;

//	wait_busy_status_GLCD ();
	write_command_GLCD (GLCD_COMM_START_LINE | vertical_offset);
//	wait_busy_status_GLCD ();

	GLCD_CS1 = LOW_GLCD;
	GLCD_CS2 = LOW_GLCD;
    
    NOP_DELAY;

}


//*********************************************
//             GLCD_write_data
//*********************************************
void GLCD_write_data (unsigned char data_to_send) {

    NOP_DELAY;
	GLCD_E = LOW_GLCD;
	NOP_DELAY;
	NOP_DELAY;
	GLCD_R_W = LOW_GLCD;
	GLCD_D_I = HIGH_GLCD;
	NOP_DELAY;
	NOP_DELAY;
	GLCD_E = HIGH_GLCD;
    NOP_DELAY;
    NOP_DELAY;
    NOP_DELAY;
    NOP_DELAY;
    NOP_DELAY;
	GLCD_DATA_WRITE = data_to_send;
	NOP_DELAY;
	NOP_DELAY;
	NOP_DELAY;
    NOP_DELAY;
    NOP_DELAY;
    NOP_DELAY;
	GLCD_E = LOW_GLCD;

	NOP_DELAY;
	GLCD_R_W = HIGH_GLCD;
	GLCD_D_I = LOW_GLCD;

	NOP_DELAY;
    NOP_DELAY;
    NOP_DELAY;
    NOP_DELAY;

	GLCD_E = HIGH_GLCD;
    NOP_DELAY;
    NOP_DELAY;
    NOP_DELAY;
    NOP_DELAY;
    NOP_DELAY;

}


//*********************************************
//             GLCD_clear
//*********************************************
void GLCD_clear (unsigned char filling_color) {
	
	unsigned char page = 0;
	unsigned char address = 0;

	GLCD_CS1 = HIGH_GLCD;
	GLCD_CS2 = LOW_GLCD;
	
	set_address_GLCD (address);

	for (page = 0; page < GLCD_NUMBER_OF_PAGES; page++) {
		set_page_GLCD (page);
		for (address = 0; address < GLCD_Y_RESOLUTION; address++) {

		//	wait_busy_status_GLCD ();
			write_data_GLCD (filling_color);
		//	wait_busy_status_GLCD ();
		}
	}

	GLCD_CS1 = LOW_GLCD;
	GLCD_CS2 = HIGH_GLCD;


	address = 0;
	set_address_GLCD (address);

	for (page = 0; page < GLCD_NUMBER_OF_PAGES; page++) {
		set_page_GLCD (page);
		
		for (address = 0; address < GLCD_Y_RESOLUTION; address++) {
		//	wait_busy_status_GLCD ();
			write_data_GLCD (filling_color);
		//	wait_busy_status_GLCD ();
		}
	}

	GLCD_CS1 = LOW_GLCD;
	GLCD_CS2 = LOW_GLCD;

}

//*********************************************
//             GLCD_read_data
//*********************************************
unsigned char GLCD_read_data (void) {

	unsigned char data_out; 

	GLCD_DATA_DIRECTION = GLCD_INPUT;

	GLCD_E = LOW_GLCD;
	NOP_DELAY;
	NOP_DELAY;
	GLCD_R_W = HIGH_GLCD;
	GLCD_D_I = HIGH_GLCD;
	NOP_DELAY;
	GLCD_E = HIGH_GLCD;
	NOP_DELAY;
    NOP_DELAY;
    NOP_DELAY;
	data_out = GLCD_DATA_READ;
	NOP_DELAY;
	NOP_DELAY;
    NOP_DELAY;
    NOP_DELAY;
	GLCD_E = LOW_GLCD;
	NOP_DELAY;
    NOP_DELAY;
    NOP_DELAY;
	GLCD_R_W = LOW_GLCD;
	GLCD_D_I = LOW_GLCD;
	
	GLCD_DATA_DIRECTION = GLCD_OUTPUT;

	return (data_out);
}

//*********************************************
//             GLCD_plot_xy
//*********************************************
void GLCD_plot_xy (unsigned char x_pos,  unsigned char y_pos, unsigned char filling_color){

	unsigned char data_byte;
	unsigned char data_mask;

	y_pos = GLCD_Y_RESOLUTION-y_pos -1;

	if (x_pos < GLCD_CONTROLLER_MAX_X) {

		GLCD_CS1 = HIGH_GLCD;
		GLCD_CS2 = LOW_GLCD;
		set_address_GLCD (x_pos);
		set_page_GLCD (y_pos >> 3);	
		//dummy read
		data_byte = read_data_GLCD ();
		data_byte = read_data_GLCD ();

		set_address_GLCD (x_pos);

	} else {
		
		GLCD_CS1 = LOW_GLCD;
		GLCD_CS2 = HIGH_GLCD;
		set_address_GLCD (x_pos - GLCD_CONTROLLER_MAX_X);
		set_page_GLCD (y_pos >> 3);	
		//dummy read
		data_byte = read_data_GLCD ();
		data_byte = read_data_GLCD ();

		set_address_GLCD (x_pos - GLCD_CONTROLLER_MAX_X);
	}

	data_mask = 0x01 << (y_pos & 0x07);	

	if (filling_color == FILLING_BLACK)
		data_byte = data_byte | data_mask;
	else
		data_byte = data_byte & ~data_mask;
	
//	wait_busy_status_GLCD ();
	write_data_GLCD (data_byte);
//	wait_busy_status_GLCD ();
	

	GLCD_CS1 = LOW_GLCD;
	GLCD_CS2 = LOW_GLCD;
}


//*********************************************
//           GLCD_draw_vertical_line
//*********************************************
void GLCD_draw_vertical_line (unsigned char x_origin, unsigned char y_origin, unsigned char length, unsigned char filling_color){

	unsigned char i;

	for (i = 0 ; i<length; i++) {
		plot_xy_GLCD (x_origin, y_origin + i, filling_color);
	}
}


//*********************************************
//          GLCD_draw_horizontal_line
//*********************************************
void GLCD_draw_horizontal_line (unsigned char x_origin, unsigned char y_origin, unsigned char length, unsigned char filling_color){

	unsigned char i;

	for (i = 0 ; i<length; i++) {
		plot_xy_GLCD (x_origin + i, y_origin, filling_color);
	}
}


//*********************************************
//             GLCD_draw_window
//*********************************************

void GLCD_draw_window (unsigned char x_origin, unsigned char y_origin, unsigned char width, unsigned char height, unsigned char filling_color){

	unsigned char i;

	draw_horizontal_line_GLCD (x_origin, y_origin, width, filling_color);
	// + 1 is required to close the window
	draw_horizontal_line_GLCD (x_origin, y_origin + height, width+1, filling_color);

	draw_vertical_line_GLCD (x_origin, y_origin, height, filling_color);
	draw_vertical_line_GLCD (x_origin + width, y_origin, height, filling_color);

}


//*********************************************
//             GLCD_draw_box
//*********************************************

void GLCD_draw_box (unsigned char x_origin, unsigned char y_origin, unsigned char width, unsigned char height, unsigned char filling_color){

	unsigned char i;

	for (i=0; i < width; i++)
		draw_vertical_line_GLCD (x_origin +i, y_origin, height, filling_color);

}

//*********************************************
//             GLCD_write_char
//*********************************************
void GLCD_write_char (unsigned char x_pos, unsigned char y_pos, unsigned char character){

	unsigned char i;
	unsigned char data;
	unsigned char x;
	unsigned char bit_value;

	for (i = 0 ; i<font_width; i++){

		data = font_5x7[character - 32][i];

		for (x = 0 ; x<= font_height; x++) {
			
			if (data & (0x80 >> x))
				bit_value = FILLING_BLACK;
			else 
				bit_value = FILLING_WHITE;

			plot_xy_GLCD (x_pos + i, y_pos + x, bit_value);
			
		}			
	}	
}

//*********************************************
//             GLCD_write_string
//*********************************************

void GLCD_write_string (unsigned char x_pos, unsigned char y_pos, unsigned char * character_array){

unsigned char i = 0;

	while (character_array[i]) {
		
		write_char_GLCD (x_pos + (i* (font_width+ GLCD_FONT_SPACE)) , y_pos,character_array[i]);
	
		i++;
	}
}


//*********************************************
//             GLCD_write_message
//*********************************************
#ifndef __XC8
void GLCD_write_message (unsigned char x_pos, unsigned char y_pos, const rom unsigned char * character) {
#endif

#ifdef __XC8
void GLCD_write_message (unsigned char x_pos, unsigned char y_pos, const unsigned char * character) {
#endif

unsigned char i = 0;

    while (character[i]) {

        write_char_GLCD (x_pos + (i*(font_width + GLCD_FONT_SPACE)), y_pos,character[i]);

        i++;
    }

}


//*********************************************
//             GLCD_draw_picture
//*********************************************
#ifndef __XC8
void GLCD_draw_picture (rom unsigned char  * picture_table){
#endif

#ifdef __XC8
void GLCD_draw_picture (const unsigned char  * picture_table){
#endif
	unsigned char page = 0;
	unsigned char address = 0;
	

	for (page = 0; page < GLCD_NUMBER_OF_PAGES; page++) {
            GLCD_CS1 = HIGH_GLCD;
            GLCD_CS2 = LOW_GLCD;

            set_address_GLCD (address);
            set_page_GLCD (page);

            for (address = 0; address < GLCD_Y_RESOLUTION; address++) {

                    write_data_GLCD (* picture_table);
                    picture_table++;
            }

            GLCD_CS1 = LOW_GLCD;
            GLCD_CS2 = HIGH_GLCD;

            address = 0;
            set_address_GLCD (address);
            set_page_GLCD (page);

            for (address = 0; address < GLCD_Y_RESOLUTION; address++) {

                    write_data_GLCD (* picture_table);
                    picture_table++;
            }

	}

	GLCD_CS1 = LOW_GLCD;
	GLCD_CS2 = LOW_GLCD;
}





//*********************************************
//             GLCD_write_integer
//*********************************************
void GLCD_write_integer (unsigned char x_pos, unsigned char y_pos, int value, char number_of_digits){

	// The array size is 5 plus end of string \0
	unsigned char convertedInt [6] = {0,0,0,0,0,0};
	
	// Index used to shift to the right the digit
	char index; 

        // Integer is converted to string
        #ifndef __XC8
            itoa (value, (char*) convertedInt);
        #endif

        #ifdef __XC8
            itoa ((char*) convertedInt, value,10);
	#endif
	
	if (number_of_digits >0 ) {
		
		convertedInt[number_of_digits] = '\0';
		
		// Shift the digit to the right removing the empty ones
		while (!(convertedInt[number_of_digits-1] <= '9' && convertedInt[number_of_digits-1] >= '0')) {
			for (index = number_of_digits-1; index > 0; index--){
				convertedInt[index] = convertedInt[index-1];
				convertedInt[index-1] = ' ';
                        }
		}
	}

	write_string_GLCD (x_pos, y_pos, convertedInt);
}


//*********************************************
//             GLCD_set_font
//*********************************************
void GLCD_set_font (unsigned char width, unsigned char height){

	font_width = width;
	font_height = height;
}


//*********************************************
//             GLCD_backlight
//*********************************************
void GLCD_backlight(unsigned char backlight) {

	GLCD_LED = backlight;
}


//*********************************************
//             GLCD_initialize
//*********************************************
void  GLCD_initialize (void) {

	GLCD_DATA_DIRECTION = GLCD_OUTPUT;

	GLCD_RST = LOW_GLCD;
	GLCD_LED = LOW_GLCD;
	GLCD_E = HIGH_GLCD;
	GLCD_R_W = HIGH_GLCD;
	GLCD_D_I = LOW_GLCD;
	GLCD_CS1 = LOW_GLCD;
	GLCD_CS2 = LOW_GLCD;
	
	delay_ms (100);

	GLCD_RST = HIGH_GLCD;
	
	backlight_GLCD (TURN_OFF_LED_GLCD);
        set_vertical_offset_GLCD (0);
	clear_GLCD (FILLING_WHITE);
	set_display_GLCD (TURN_ON_DISPLAY);
	set_font_GLCD (GLCD_FONT_WIDTH_5, GLCD_FONT_HEIGTH_7);

}



