/*******************************************************************************

Author : Mauro Laurenti
Version : 1.1
Created on Date : 07/02/2014
Last update     : 15/12/2014

CopyRight 2006-2014 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/

#ifdef __XC8
    #include <xc.h>
        #ifndef _PIC18
            #error The MCP2300x Library supports only PIC18 devices
        #endif
#endif

#include "MCP2300x.h"


//************************************************************
//         initialize_MC2300x function implementation
//************************************************************
void MCP2300x_initialize (unsigned char crystal_frequency_MHz, unsigned int baud_rate_KHz){

    OpenI2C(MASTER, SLEW_ON);

    SSPADD = (((crystal_frequency_MHz *1000)/4)/baud_rate_KHz)-1;
}

//************************************************************
//         set_register_MCP2300x function implementation
//************************************************************

signed char MCP2300x_set_register (unsigned char device_address, unsigned char register_add,  unsigned char data ) {

    unsigned char control_byte = 0;

    control_byte = INTERNAL_ADDRESS | (device_address <<1);

    return (EEByteWrite (control_byte,register_add,data));
}



//************************************************************
//           get_register_MCP2300x function implementation
//************************************************************

unsigned char MCP2300x_get_register (unsigned char device_address, unsigned char register_add){
	
    unsigned char control_byte = 0;
    control_byte = INTERNAL_ADDRESS | (device_address <<1);

    return (EERandomRead (control_byte,register_add));
}


//************************************************************
//         set_port_direction_MCP2300x function implementation
//************************************************************
signed char MCP2300x_set_port_direction (unsigned char device_address, unsigned char direction) {

    return (set_register_MCP2300x (device_address, MCP2300x_IODIR, direction));
}


//************************************************************
//         set_port_value_MCP2300x function implementation
//************************************************************
signed char MCP2300x_set_port_value (unsigned char device_address, unsigned char value){

    return (set_register_MCP2300x (device_address, MCP2300x_GPIO, value));
}


//************************************************************
//         set_port_polarity_MCP2300x function implementation
//************************************************************
signed char MCP2300x_set_port_polarity (unsigned char device_address, unsigned char value){

    return (set_register_MCP2300x (device_address, MCP2300x_IPOL, value));
}


//************************************************************
// set_port_pull_up_resistor_MCP2300x function implementation
//************************************************************
signed char MCP2300x_set_port_pull_up_resistor (unsigned char device_address, unsigned char value){

    return (set_register_MCP2300x (device_address, MCP2300x_GPPU, value));
}


//************************************************************
//         get_port_value_MCP2300x function implementation
//************************************************************
unsigned char MCP2300x_get_port_value (unsigned char device_address){

    return (get_register_MCP2300x (device_address, MCP2300x_GPIO));
}



//************************************************************
//         set_port_interrupt_MCP2300x function implementation
//************************************************************
signed char MCP2300x_set_port_interrupt (unsigned char device_address, unsigned char value){

    signed char return_value = 0;

    return_value =  set_register_MCP2300x (device_address, MCP2300x_GPINTEN, value);

    //This dummy read of the GPIO register is used to clean the interupt flags
    get_register_MCP2300x (device_address, MCP2300x_GPIO);

    return (return_value);

}

//************************************************************
//         set_port_interrupt_configuration_MCP2300x function implementation
//************************************************************
signed char MCP2300x_set_port_configuration (unsigned char device_address, unsigned char value){
    return (set_register_MCP2300x (device_address, MCP2300x_IOCON, value));
}

//************************************************************
// set_interrupt_compare_value_MCP2300x function implementation
//************************************************************
signed char MCP2300x_set_interrupt_compare_value (unsigned char device_address, unsigned char value){

    return (set_register_MCP2300x (device_address, MCP2300x_DEFVAL, value));
}


//************************************************************
// set_interrupt_compare_enable_MCP2300x function implementation
//************************************************************
signed char MCP2300x_set_interrupt_compare_enable (unsigned char device_address, unsigned char value){

    return (set_register_MCP2300x (device_address, MCP2300x_INTCON, value));
}


//************************************************************
// get_port_interrupt_flag_MCP2300x function implementation
//************************************************************
unsigned char MCP2300x_get_port_interrupt_flag (unsigned char device_address){

    return (get_register_MCP2300x (device_address, MCP2300x_INTF));
}


//************************************************************
// get_port_interrupt_capture_MCP2300x function implementation
//************************************************************
unsigned char MCP2300x_get_port_interrupt_capture (unsigned char device_address){
    return (get_register_MCP2300x (device_address, MCP2300x_INTCAP));
}


