/*******************************************************************************

Author : Mauro Laurenti
Version : 1.2
Created on Date : 4/9/2006
Last update     : 15/12/2014

CopyRight 2006-2014 all rights are reserved

********************************************************
SOFTWARE LICENSE AGREEMENT
********************************************************

The usage of the supplied software imply the acceptance of the following license.

The software supplied herewith by Mauro Laurenti (the Author) is intended for
use solely and exclusively on Microchip PIC Microcontroller (registered mark).
The software is owned by the Author, and is protected under applicable
copyright laws. All rights are reserved.
Any use in violation of the foregoing restrictions may subject the
user to criminal sanctions under applicable laws, as well as to civil liability
for the breach of the terms and conditions of this license.
Commercial use is forbidden without a written acknowledgement with the Author.
Personal or educational use is allowed if the application containing the
following software doesn't aim to commercial use or monetary earning of any kind.

THIS SOFTWARE IS PROVIDED IN AN "AS IS" CONDITION. NO WARRANTIES,
WHETHER EXPRESS, IMPLIED OR STATUTORY, INCLUDING, BUT NOT LIMITED
TO, IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A
PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. THE AUTHOR SHALL NOT,
IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL OR
CONSEQUENTIAL DAMAGES, FOR ANY REASON WHATSOEVER.

*******************************************************************************/

#ifdef __XC8
    #include <xc.h>
        #ifndef _PIC18
            #error The PCF8563 Library supports only PIC18 devices
        #endif
#endif


#include "PCF8563.h"

//************************************************************
//         PCF8563_initialize function implementation
//************************************************************
void PCF8563_initialize (unsigned char crystal_frequency_MHz, unsigned int baud_rate_KHz){

    OpenI2C(MASTER, SLEW_ON);

    SSPADD = (((crystal_frequency_MHz *1000)/4)/baud_rate_KHz)-1;
}


//************************************************************
//             RTCC_set_seconds function implementation
//************************************************************
signed char RTCC_set_seconds (unsigned char seconds) {
	
	return (EEByteWrite (RTCC_WRITE_ADD, RTCC_SECONDS_ADDR,seconds));
	
}


//************************************************************
//             RTCC_get_seconds function implementation
//************************************************************
unsigned char RTCC_get_seconds (void) {
	
	unsigned char seconds;
	
	seconds = EERandomRead (RTCC_WRITE_ADD, RTCC_SECONDS_ADDR);
	
	// I set to 0 the not significant bits
	seconds = seconds & 0b01111111;	
	return (seconds);
}



//************************************************************
//             RTCC_set_minutes function implementation
//************************************************************
signed char RTCC_set_minutes (unsigned char minutes) {
	
	return (EEByteWrite (RTCC_WRITE_ADD, RTCC_MINUTES_ADDR,minutes));
	
}


//************************************************************
//             RTCC_get_minutes function implementation
//************************************************************
unsigned char RTCC_get_minutes (void) {
	
	unsigned char minutes;
	minutes = EERandomRead (RTCC_WRITE_ADD, RTCC_MINUTES_ADDR);
	
	// I set to 0 the not significant bits
	minutes = minutes & 0b01111111;	
	return (minutes);
}


//************************************************************
//             RTCC_set_hours function implementation
//************************************************************
signed char RTCC_set_hours (unsigned char hours) {
		
	return (EEByteWrite (RTCC_WRITE_ADD, RTCC_HOURS_ADDR,hours));
}


//************************************************************
//             RTCC_get_hours function implementation
//************************************************************
unsigned char RTCC_get_hours (void)

{	unsigned char hours;
	hours = EERandomRead (RTCC_WRITE_ADD, RTCC_HOURS_ADDR);
	
	// I set to 0 the not significant bits 
	hours = hours & 0b00111111;	
	return (hours);
}


//************************************************************
//             RTCC_get_time_seconds function implementation
//************************************************************
unsigned char* RTCC_get_time_seconds (void) {
	
	static unsigned char time[9];
	unsigned char value;

	value = get_hours_RTCC();	
	
	// with +48 I convert the number in ASCII number
	time[1] = (value & 0b00001111)+48; 
	time[0] = (value >> 4)+48;
	
	time[2] = ':';
	
	value = get_minutes_RTCC();	
	time[4] = (value & 0b00001111)+48;
	time[3] = (value >> 4)+48;
	
	time[5] = '.';

	value = get_seconds_RTCC();	
	time[7] = (value & 0b00001111)+48;
	time[6] = (value >> 4)+48;
	
	time[8] = '\0';
		
	return (time);
	
}


//************************************************************
//             RTCC_get_time function implementation
//************************************************************
unsigned char* RTCC_get_time (void) {
	
	static unsigned char time[6];
	unsigned char value;

	value = get_hours_RTCC();	
	
	// with +48 I convert the number in ASCII number
	time[1] = (value & 0b00001111)+48; 
	time[0] = (value >> 4)+48;
	
	time[2] = ':';
	
	value = get_minutes_RTCC();	
	time[4] = (value & 0b00001111)+48;
	time[3] = (value >> 4)+48;
	
	
	time[5] = '\0';
		
	return (time);	
}


//************************************************************
//             RTCC_set_days function implementation
//************************************************************
signed char RTCC_set_days (unsigned char days) {
	
	return (EEByteWrite (RTCC_WRITE_ADD, RTCC_DAYS_ADDR,days));
}


//************************************************************
//             RTCC_get_days function implementation
//************************************************************
unsigned char RTCC_get_days (void) {
	
	unsigned char days;
	days = EERandomRead (RTCC_WRITE_ADD, RTCC_DAYS_ADDR);
	
	// I set to 0 the not significant bits
	days = days & 0b00111111;	
	return (days);
}


//************************************************************
//             RTCC_set_day_of_the_week function implementation
//************************************************************
signed char RTCC_set_day_of_the_week (unsigned char day_of_the_week) {
	
	return (EEByteWrite (RTCC_WRITE_ADD, RTCC_DAY_WEEK_ADDR,day_of_the_week));
}


//************************************************************
//             RTCC_get_week_days function implementation
//************************************************************
unsigned char RTCC_get_week_days (void){
	
	unsigned char day_of_the_week;
	day_of_the_week = EERandomRead (RTCC_WRITE_ADD, RTCC_DAY_WEEK_ADDR);
	
	// I set to 0 the not significant bits
	day_of_the_week = day_of_the_week & 0b00000111;	
	return (day_of_the_week);
}


//************************************************************
//             RTCC_set_months function implementation
//************************************************************
signed char RTCC_set_months (unsigned char months) {
	
	return (EEByteWrite (RTCC_WRITE_ADD, RTCC_MONTHS_ADDR,months));
	
}
																				

//************************************************************
//             RTCC_get_months function implementation
//************************************************************
unsigned char RTCC_get_months (void) {
	
	unsigned char months;
	months = EERandomRead (RTCC_WRITE_ADD, RTCC_MONTHS_ADDR);
	
	// I set to 0 the not significant bits
	months = months & 0b00011111;	
	return (months);
}


//************************************************************
//             RTCC_set_years function implementation
//************************************************************
signed char RTCC_set_years (unsigned char years) {
	
	return (EEByteWrite (RTCC_WRITE_ADD, RTCC_YEARS_ADDR,years));
}


//************************************************************
//             RTCC_get_years function implementation
//************************************************************
unsigned char RTCC_get_years (void) {
	
	unsigned char years;
	years = EERandomRead (RTCC_WRITE_ADD, RTCC_YEARS_ADDR);
	return (years);
}


//************************************************************
//             RTCC_get_date function implementation
//************************************************************
unsigned char* RTCC_get_date (void) {
	
	static unsigned char date[9];
	unsigned char value;

	value = get_days_RTCC();	
	
	// with +48 I convert the number in ASCII number
	date[1] = (value & 0b00001111)+48; 
	date[0] = (value >> 4)+48;
	
	date[2] = '/';
	
	value = get_months_RTCC();	
	date[4] = (value & 0b00001111)+48;
	date[3] = (value >> 4)+48;
	
	date[5] = '/';


	value = get_years_RTCC();	
	date[7] = (value & 0b00001111)+48;
	date[6] = (value >> 4)+48;
	
	date[8] = '\0';
		
	return (date);
	
}


//************************************************************
//             RTCC_set_minutes_alarm function implementation
//************************************************************
signed char RTCC_set_minutes_alarm (unsigned char minutes, unsigned char alarm_enable) {
	
	//I activate AE if required
	minutes = minutes + alarm_enable; 
	
	return (EEByteWrite (RTCC_WRITE_ADD, RTCC_MINUTS_ALARM_ADDR,minutes));
	
}


//************************************************************
//             RTCC_set_hours_alarm function implementation
//************************************************************
signed char RTCC_set_hours_alarm (unsigned char hours, unsigned char alarm_enable) {
	
	//I activate AE if required
	hours = hours + alarm_enable; 
	return (EEByteWrite (RTCC_WRITE_ADD, RTCC_HOURS_ALARM_ADDR,hours));
}


//************************************************************
//             RTCC_set_days_alarm function implementation
//************************************************************
signed char RTCC_set_days_alarm (unsigned char days, unsigned char alarm_enable) {
	
	//I activate AE if required
	days = days + alarm_enable; 
	return (EEByteWrite (RTCC_WRITE_ADD, RTCC_DAYS_ALARM_ADDR,days));
	
}


//************************************************************
//   RTCC_set_day_of_the_week_alarm function implementation
//************************************************************
signed char RTCC_set_day_of_the_week_alarm (unsigned char day_of_the_week_alarm, unsigned char alarm_enable) {
		
	//I activate AE if required
	day_of_the_week_alarm = day_of_the_week_alarm + alarm_enable; 
	return (EEByteWrite (RTCC_WRITE_ADD, RTCC_DAY_WEEK_ALARM_ADDR,day_of_the_week_alarm));
	
}


//************************************************************
//             RTCC_enable_alarm_interrupt function implementation
//************************************************************
signed char RTCC_enable_alarm_interrupt (void) {
	
	return (EEByteWrite (RTCC_WRITE_ADD, RTCC_CONTROL_REG_2_ADDR,0x02));
	
}


//************************************************************
//      RTCC_disable_alarm_interrupt function implementation
//************************************************************
signed char RTCC_disable_alarm_interrupt (void) {
	
	return (EEByteWrite (RTCC_WRITE_ADD, RTCC_CONTROL_REG_2_ADDR,0x00));
	
}


//************************************************************
//       RTCC_is_alarm_ON function implementation
//************************************************************
unsigned char RTCC_is_alarm_ON (void) {
	
	unsigned char value;
	value = EERandomRead (RTCC_WRITE_ADD, RTCC_CONTROL_REG_2_ADDR);
	
	// Just AF bit is controlled 
	if (value & 0x08) {
		
		value = value & 0xF7;	
		
		// I clean AF bit without canging the other bits
		EEByteWrite (RTCC_WRITE_ADD, RTCC_CONTROL_REG_2_ADDR,value);
		return (1);	
	
	} else {
		return (0);	
	}
		
}


//************************************************************
//         RTCC_increment_minutes function implementation
//************************************************************
signed char RTCC_increment_minutes (void) {
	
	unsigned char minutes;
	signed char error;
	
	// Read the current minutes
	minutes = get_minutes_RTCC ();
	
	// Increment the minutes
	minutes ++;
	
	// Check the minute limits

	if ((minutes&0x0F) > (unsigned char) 9 ) {		
		minutes &= 0xF0;
		minutes += 0x10;
	}
	
	if (minutes == (unsigned char) RTCC_MAX_MINUTES) {
		
		minutes = 0;
	}
	
	// Update the minutes
	error = set_minutes_RTCC (minutes); 
	
	return (error);	
	
}


//************************************************************
//           RTCC_increment_hours function implementation
//************************************************************
signed char RTCC_increment_hours (void) {
	
	unsigned char hours;
	signed char error;
	
	// Read the current hours
	hours = get_hours_RTCC ();
	
	// Increment the hours
	hours ++;
	
	// Check the hour limits
	
	if ((hours&0x0F) > (unsigned char) 9 ) {		
		hours &= 0xF0;
		hours += 0x10;
	}
	
	if (hours == (unsigned char) RTCC_MAX_HOURS) {
		
		hours = 0;
	}
	
	// Update the hours
	error = set_hours_RTCC (hours); 
	
	return (error);
	
}


//************************************************************
//           RTCC_increment_years function implementation
//************************************************************
signed char RTCC_increment_years (void) {
	
	unsigned char years;
	signed char error;
	
	// Read the current years
	years = get_years_RTCC ();
	
	// Increment the years
	years ++;
	
	// Check the year limits
	
	if ((years&0x0F) > (unsigned char) 9 ) {		
		years &= 0xF0;
		years += 0x10;
	}
	
	if (years == (unsigned char) RTCC_MAX_YEARS) {
		
		years = 0;
	}
	
	// Update the years
	error = set_years_RTCC (years); 
	
	return (error);
	
}


//************************************************************
//           RTCC_increment_months function implementation
//************************************************************
signed char RTCC_increment_months (void) {
	
	unsigned char months;
	signed char error;
	
	// Read the current months
	months = get_months_RTCC ();
	
	// Increment the months
	months ++;
	
	// Check the month limits
	
	if ((months&0x0F) > (unsigned char) 9 ) {		
		months &= 0xF0;
		months += 0x10;
	}
	
	if (months == (unsigned char) RTCC_MAX_MONTHS) {
		
		months = 1;
	}
	
	// Update the months
	error = set_months_RTCC (months); 
	
	return (error);
	
}

//************************************************************
//           RTCC_increment_days function implementation
//************************************************************
signed char RTCC_increment_days (void) {
	
	unsigned char days;
	signed char error;
	
	// Read the current days
	days = get_days_RTCC ();
	
	// Increment the days
	days ++;
	
	// Check the day limits
	
	if ((days&0x0F) > (unsigned char) 9 ) {		
		days &= 0xF0;
		days += 0x10;
	}
	
	if (days == (unsigned char) RTCC_MAX_DAYS) {
		
		days = 1;
	}
	
	// Update the days
	error = set_days_RTCC (days); 
	
	return (error);	
}

